"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createHttpCNetlifyHandler = void 0;
const stream_1 = require("stream");
const server_1 = require("@httpc/server");
let handler;
let initialized = false;
let initializing;
function createHttpCNetlifyHandler(options) {
    if (!handler) {
        const server = (0, server_1.createHttpCServer)(options);
        handler = server.getHttpCRequestProcessor();
    }
    return async (event, context) => {
        if (options.kit && !initialized) {
            if (!initializing) {
                initializing = Promise.resolve().then(async () => {
                    var _a;
                    const lib = "@httpc/kit";
                    const { initializeContainer } = await (_a = lib, Promise.resolve().then(() => __importStar(require(_a))));
                    await initializeContainer();
                    initialized = true;
                    initializing = undefined;
                });
            }
            await initializing;
        }
        const request = createRequest(event);
        const response = createResponse();
        await Promise.allSettled([
            handler(request, response),
            new Promise((resolve, reject) => {
                response.on("close", resolve);
                response.on("error", reject);
            })
        ]);
        const buffer = response.getBuffer();
        return {
            statusCode: response.statusCode,
            headers: response.getHeaders(),
            body: buffer?.toString("utf8")
        };
    };
}
exports.createHttpCNetlifyHandler = createHttpCNetlifyHandler;
function createRequest(event) {
    let body;
    if (event.body) {
        if (typeof event.body === "string") {
            body = stream_1.Readable.from(Buffer.from(event.body, event.isBase64Encoded ? "base64" : "utf8"));
        }
        else {
            console.warn("Body (%s): %s", typeof event.body, event.body?.toString());
            throw new Error("Unknown body type");
        }
    }
    else {
        body = stream_1.Readable.from([]);
    }
    return Object.assign(body, {
        url: event.rawUrl,
        method: event.httpMethod,
        headers: event.headers,
    });
}
function createResponse() {
    const writable = new BufferedWritable();
    let _statusCode = 200;
    let _headers = {};
    return Object.assign(writable, {
        get statusCode() { return _statusCode; },
        set statusCode(value) { _statusCode = value; },
        getHeaders() {
            return _headers;
        },
        setHeader(name, value) {
            _headers[name] = value;
        },
        writeHead(statusCode, headers) {
            _statusCode = statusCode;
            _headers = { ..._headers, ...headers };
            return this;
        }
    });
}
class BufferedWritable extends stream_1.Writable {
    getBuffer() {
        return this._buffer;
    }
    _write(chunk, encoding, callback) {
        const newChunk = Buffer.from(chunk, encoding);
        if (!this._buffer) {
            this._buffer = chunk;
        }
        else {
            this._buffer = Buffer.concat([this._buffer, newChunk]);
        }
        callback();
    }
}
